#include <windows.h>


#define RENTRANT 0
#define KbMedFile KbNEZFile
#include "kmnezext.h"

#include "kmpsdk/kmp_pi.h"
#include "nezfile.h"

#if !RENTRANT
static LONG playcount = 0;
#endif

static HKMP WINAPI kmp_Open(const char *cszFileName, SOUNDINFO *pInfo)
{
	if (!pInfo) return 0;
#if !RENTRANT
	if (playcount) return 0;
#endif
	KbMedFile *pFile = new KbMedFile;
	if(!pFile || !pFile->Open(cszFileName, pInfo))
	{
		if (pFile) delete pFile;
		return 0;
	}
#if !RENTRANT
	InterlockedIncrement(&playcount);
#endif
	return pFile;
}

static void WINAPI kmp_Close(HKMP hKMP)
{
	if (hKMP)
	{
		delete (KbMedFile*)hKMP;
#if !RENTRANT
		InterlockedDecrement(&playcount);
#endif
	}
}

static DWORD WINAPI kmp_Render(HKMP hKMP, BYTE* Buffer, DWORD dwSize)
{
    if(!hKMP)return 0;
    KbMedFile *pFile = (KbMedFile*)hKMP;
    return pFile->Render(Buffer, dwSize);
}

static DWORD WINAPI kmp_SetPosition(HKMP hKMP, DWORD dwPos)
{
    if(!hKMP)return 0;
    KbMedFile *pFile = (KbMedFile*)hKMP;
    pFile->SetPosition(dwPos);
    return 0;
}

KMPMODULE* WINAPI kmp_GetTestModule(void)
{
	static KMPMODULE Mod = {
		KMPMODULE_VERSION,//dwVersion
		KMPMODULE_PLUGIN_VERSION,
		KMPMODULE_PLUGIN_COPYRIGHT,
		KMPMODULE_PLUGIN_NAME,
		ppszSupportExts,	//ppszSupportExts
		RENTRANT,			//dwRentrant
		NULL,				//Init
		NULL,				//Deinit
		kmp_Open,			//Open
		NULL,				//OpenFromBuffer
		kmp_Close,			//Close
		kmp_Render,			//Render
		kmp_SetPosition		//SetPosition
	};
	return &Mod;
}
